// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/*-
 * Copyright (c) 2016 elementary LLC. (https://elementary.io)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Corentin Noël <corentin@elementary.io>, Andrei-Costin Zisu <matzipan@gmail.com>
 */

public class Scratch.Plugins.FolderManager.FolderItem : Item {
    private GLib.FileMonitor monitor;
    private bool children_loaded = false;

    public FolderItem (Scratch.Plugins.FolderManager.File file, Scratch.Plugins.FolderManager.FileView view) requires (file.is_valid_directory) {
        Object (file: file, view: view);
    }

    construct {
        if (file.children.length () > 0) {
            add (new Granite.Widgets.SourceList.Item ("")); // dummy
        }

        toggled.connect (() => {
            if (expanded && n_children <= 1) {
                clear ();
                add_children ();
                children_loaded = true;
            }
        });
        
        try {
            monitor = file.file.monitor_directory (GLib.FileMonitorFlags.NONE);
            monitor.changed.connect (on_changed);
        } catch (GLib.Error e) {
            warning (e.message);
        }
    }

    public override Gtk.Menu? get_context_menu () {
        var menu = new Gtk.Menu ();
        if (parent == view.root) {
            var item = new Gtk.MenuItem.with_label (_("Close Folder"));
            item.activate.connect (() => {
                monitor.cancel ();
                parent.remove (this);
            });
            menu.append (item);
        } else {
            var item = new Gtk.MenuItem.with_label (_("Open"));
            item.activate.connect (() => { view.open_folder (file.path); });
            menu.append (item);
        }

        var rename_item = new Gtk.MenuItem.with_label (_("Rename"));
        rename_item.activate.connect (() => view.start_editing_item (this));
        menu.append (rename_item);

        var new_file_item = new Gtk.MenuItem.with_label (_("Add file"));
        new_file_item.activate.connect (() => add_file ());
        menu.append (new_file_item);

        var new_folder_item = new Gtk.MenuItem.with_label (_("Add folder"));
        new_folder_item.activate.connect(() => add_folder ());
        menu.append (new_folder_item);

        var delete_item = new Gtk.MenuItem.with_label (_("Move to Trash"));
        delete_item.activate.connect (() => do_remove ());
        menu.append (delete_item);

        menu.show_all ();
        return menu;
    }

    internal void add_children () {
        foreach (var child in file.children) {
            if (child.is_valid_directory) {
                var item = new FolderItem (child, view);
                add (item);
            } else if (child.is_valid_textfile) {
                var item = new FileItem (child, view);
                add (item);
            }
        }
    }

    private void on_changed (GLib.File source, GLib.File? dest, GLib.FileMonitorEvent event) {
        if (!children_loaded) {
            return;
        }

        switch (event) {
            case GLib.FileMonitorEvent.DELETED:
                var children_tmp = new Gee.ArrayList<Granite.Widgets.SourceList.Item> ();
                children_tmp.add_all (children);
                foreach (var item in children_tmp) {
                    if ((item as Item).path == source.get_path ()) {
                        remove (item);
                    }
                }

                break;
            case GLib.FileMonitorEvent.CREATED:
                if (source.query_exists () == false) {
                    return;
                }

                var file = new File (source.get_path ());
                var exists = false;
                foreach (var item in children) {
                    if ((item as Item).path == file.path) {
                        exists = true;
                    }
                }

                if (!exists) {
                    if (file.is_valid_textfile) {
                        add (new FileItem (file, view));
                    } else if (file.is_valid_directory) {
                        add (new FolderItem (file, view));
                    }
                }

                break;
        }
    }

    private void add_folder () {
        var child = file.file.get_child (_("untitled folder"));

        var n = 1;
        while (child.query_exists ()) {
            child = file.file.get_child (_("untitled folder (%d)").printf (n));
            n++;
        }

        try {
            child.make_directory ();

            if (children_loaded) {
                var item = new FolderItem (new File (child.get_path ()), view);
                add (item);
                view.start_editing_item (item);
            }
        } catch (Error e) {
            warning (e.message);
        }

        
    }

    private void add_file () {
        var child = file.file.get_child (_("new file"));

        var n = 1;
        while (child.query_exists ()) {
            child = file.file.get_child (_("new file (%d)").printf (n));
            n++;
        }

        try {
            child.create (FileCreateFlags.NONE);

            if (children_loaded) {
                var item = new FileItem (new File (child.get_path ()), view);
                add (item);
                view.start_editing_item (item);
            }
        } catch (Error e) {
            warning (e.message);
        }
    }
}   
